<?php

namespace BitWasp\Bitcoin\Script;

use BitWasp\Bitcoin\Bitcoin;
use BitWasp\Bitcoin\Math\Math;
use BitWasp\Bitcoin\Script\Factory\InputScriptFactory;
use BitWasp\Bitcoin\Script\Factory\OutputScriptFactory;
use BitWasp\Bitcoin\Script\Factory\P2shScriptFactory;
use BitWasp\Bitcoin\Script\Factory\ScriptCreator;
use BitWasp\Bitcoin\Script\Factory\ScriptInfoFactory;
use BitWasp\Buffertools\Buffer;

class ScriptFactory
{
    /**
     * @param Buffer|null $buffer
     * @param Opcodes|null $opcodes
     * @param Math|null $math
     * @return ScriptCreator
     */
    public static function create(Buffer $buffer = null, Opcodes $opcodes = null, Math $math = null)
    {
        return new ScriptCreator($math ?: Bitcoin::getMath(), $opcodes ?: new Opcodes(), $buffer);
    }

    /**
     * @return InputScriptFactory
     */
    public static function scriptSig()
    {
        return new InputScriptFactory();
    }

    /**
     * @return OutputScriptFactory
     */
    public static function scriptPubKey()
    {
        return new OutputScriptFactory();
    }

    /**
     * @return P2shScriptFactory
     */
    public static function p2sh()
    {
        return new P2shScriptFactory(self::scriptPubKey());
    }

    /**
     * @param ScriptInterface $script
     * @param ScriptInterface|null $redeemScript
     * @return ScriptInfo\ScriptInfoInterface
     */
    public static function info(ScriptInterface $script, ScriptInterface $redeemScript = null)
    {
        return (new ScriptInfoFactory())->load($script, $redeemScript);
    }

     /**
     * @param Buffer|string $string
     * @return Script
     */
    public static function fromHex($string)
    {
        return self::create($string instanceof Buffer ? $string : Buffer::hex($string))->getScript();
    }
}
